import datetime
import dateutil.parser
import os.path

from changelog_generator.log_handlers import logger
from changelog_generator.calls import (
    get_closed_issues_for_project,
    get_commits_since_date,
    get_last_commit_date,
    get_last_tagged_release_date,
)


def generate_changelog(cli_args: dict) -> str:
    # Get the date of the last commit
    last_commit = get_last_commit_date(cli_args)

    closed_issues_since_last_tag = get_closed_issues_since_last_tag(cli_args)

    # Get any commits since that date
    new_commits = get_commits_since_date(last_commit, cli_args)

    # Get the current date so that we can add it to the CHANGELOG.md document
    date = datetime.datetime.now()
    current_date = date.strftime("%Y-%m-%d")

    # Determine whether a CHANGELOG.md file already exists
    if os.path.isfile("CHANGELOG.md"):
        if cli_args["changelog"] == "Y":
            with open("CHANGELOG.md", "r") as original_changelog:
                original_changelog_data = original_changelog.read()
                with open("CHANGELOG.md", "w") as modified_changelog:
                    modified_changelog.write(
                        f"## v{cli_args['version']} ({current_date})\n"
                    )
                    [
                        modified_changelog.write(
                            f"* {commit['committed_date'][:10]} - {y} \n"
                        )
                        for commit in new_commits
                        for y in commit["message"].split("\n")
                    ]

                    if closed_issues_since_last_tag:
                        modified_changelog.write(f"\n### Closed Issues\n")
                        [
                            modified_changelog.write(
                                f"* {closed_issue['title']}"
                            )
                            for closed_issue in closed_issues_since_last_tag
                        ]
                    modified_changelog.write(f"\n")
                    modified_changelog.write(original_changelog_data)
                    return "CHANGELOG.md updated successfully"
        else:
            logger.info("Existing CHANGELOG.md found but not specified")
            logger.info("Writing CHANGELOG_generated.md as a result...")
            with open("CHANGELOG_generated.md", "w") as changelog:
                changelog.write(f"## v{cli_args['version']} ({current_date})\n")
                [
                    changelog.write(
                        f"* {commit['committed_date'][:10]} - {y} \n"
                    )
                    for commit in new_commits
                    for y in commit["message"].split("\n")
                ]

                if closed_issues_since_last_tag:
                    changelog.write(f"\n### Closed Issues\n")
                    [
                        changelog.write(f"* {closed_issue['title']}")
                        for closed_issue in closed_issues_since_last_tag
                    ]
            return "CHANGELOG_generated.md written successfully"
    else:
        logger.info("No CHANGELOG.md found and no CHANGELOG.md specified...")
        logger.info("Writing CHANGELOG.md as a result...")
        with open("CHANGELOG.md", "w") as changelog:
            changelog.write(f"## v{cli_args['version']} ({current_date})\n")
            [
                changelog.write(f"* {commit['committed_date'][:10]} - {y} \n")
                for commit in new_commits
                for y in commit["message"].split("\n")
            ]
            if closed_issues_since_last_tag:
                changelog.write(f"\n### Closed Issues\n")
                [
                    changelog.write(f"* {closed_issue['title']}")
                    for closed_issue in closed_issues_since_last_tag
                ]
        return "New CHANGELOG.md file written successfully"


def get_closed_issues_since_last_tag(cli_args: dict) -> list:
    last_tagged_release_date = get_last_tagged_release_date(cli_args)

    closed_issues = get_closed_issues_for_project(cli_args)

    closed_issues_since_tag = []
    for issue in closed_issues:
        logger.info(issue)
        if dateutil.parser.parse(issue["closed_at"]) > dateutil.parser.parse(
            last_tagged_release_date
        ):
            closed_issues_since_tag.append(
                {"closed_at": issue["closed_at"], "title": issue["title"]}
            )

    return closed_issues_since_tag



import logging

from logging.config import dictConfig


logging_config = dict(
    version=1,
    formatters={
        "f": {"format": "%(asctime)s %(name)-12s %(levelname)-8s %(message)s"}
    },
    handlers={
        "h": {
            "class": "logging.StreamHandler",
            "formatter": "f",
            "stream": "ext://sys.stdout",
            "level": logging.DEBUG,
        }
    },
    root={"handlers": ["h"], "level": logging.DEBUG},
)

dictConfig(logging_config)

logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())

logger.debug("Logging initialised...")



from argparse import ArgumentParser

from changelog_generator.generator import generate_changelog


def process_arguments() -> dict:
    parser = ArgumentParser(prog="changegen")
    parser.add_argument(
        "-i",
        "--ip",
        dest="ip",
        help="specify IP address of GitLab repository - should include \
              protocol (http/s)",
        required=True,
    )
    parser.add_argument(
        "-a",
        "--api",
        dest="api",
        help="specify GitLab API version",
        choices=["1", "2", "3", "4"],
        default="4",
    )
    parser.add_argument(
        "-g",
        "--group",
        dest="group",
        help="specify GitLab group",
        required=True,
    )
    parser.add_argument(
        "-p",
        "--project",
        dest="project",
        help="specify GitLab project",
        required=True,
    )
    parser.add_argument(
        "-b",
        "--branches",
        nargs=2,
        dest="branches",
        help="specify GitLab branches to compare",
        required=True,
    )
    parser.add_argument(
        "-c",
        "--changelog",
        dest="changelog",
        help="specify whether an existing CHANGELOG.md exists",
        choices=["Y", "N"],
        default="N",
    )
    parser.add_argument(
        "-v",
        "--version",
        dest="version",
        help="specify version number",
        required=True,
    )
    parser.add_argument(
        "-t",
        "--token",
        dest="token",
        help="gitlab personal token for auth",
        required=False,
    )
    parser.add_argument(
        "-s",
        "--ssl",
        dest="ssl",
        help="specify whether or not to enable ssl",
        required=False,
        default=True,
        type=lambda x: (str(x).lower() in ["false", "2", "no"]),
    )

    args = parser.parse_args()

    return {
        "ip_address": args.ip,
        "api_version": args.api,
        "project_group": args.group,
        "project": args.project,
        "branch_one": args.branches[0],
        "branch_two": args.branches[1],
        "version": args.version,
        "changelog": args.changelog,
        "token": args.token,
        "ssl": args.ssl,
    }


def main():
    generate_changelog(process_arguments())


if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="gitlab-changelog-generator",
    version="1.0.6",
    author="Stuart McColl",
    author_email="it@stuartmccoll.co.uk",
    description="A small command line utility for generating CHANGELOG.md "
    + "files from GitLab repository commits",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords="gitlab changelog python",
    url="https://github.com/stuartmccoll/gitlab-changelog-generator",
    entry_points={
        "console_scripts": ["changegen=changelog_generator.entry_point:main"]
    },
    packages=setuptools.find_packages(),
    install_requires=["requests", "python-dateutil"],
    tests_require=["unittest", "mock"],
    classifiers=(
        "Environment :: Console",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3 :: Only",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
)



import datetime
import logging
import requests
import sys

from dateutil import parser


logger = logging.getLogger(__name__)


def get_last_commit_date(cli_args: dict) -> str:
    """
    Queries a specified GitLab API and returns the date of the most
    recent commit.
    """
    request_url = f"{cli_args['ip_address']}/api/v{cli_args['api_version']}/projects/" f"{cli_args['project_group']}%2F{cli_args['project']}/repository/commits/{cli_args['branch_one']}"
    logger.info(f"Requesting last commit date with URL: {request_url}")
    try:
        response = requests.get(
            request_url,
            headers={
                "PRIVATE-TOKEN": cli_args["token"]
                if "token" in cli_args
                else None
            },
            verify=cli_args["ssl"],
        )
        logger.info(response.status_code)
        response.raise_for_status()
    except requests.exceptions.HTTPError as ex:
        logger.error(
            f"{get_last_commit_date.__name__} call to GitLab API failed with HTTPError: {ex}"
        )
        sys.exit(1)
    except requests.exceptions.ConnectionError as ex:
        logger.error(
            f"{get_last_commit_date.__name__} call to GitLab API failed with ConnectionError: {ex}"
        )
        sys.exit(1)

    logger.debug(response.status_code)
    logger.debug(response.json())

    response_json = response.json()

    commit_date = datetime.datetime.strptime(
        response_json["committed_date"][:-6], "%Y-%m-%dT%H:%M:%S.%f"
    ) + datetime.timedelta(seconds=1)

    commit_date = parser.parse(
        response_json["committed_date"]
    ) + datetime.timedelta(microseconds=1)

    return datetime.datetime.strftime(commit_date, "%Y-%m-%dT%H:%M:%S.%f")


def get_closed_issues_for_project(cli_args: dict) -> dict:
    """
    Queries a specified GitLab API and returns a list containing
    the titles and URLs of closed issues since a given date.
    """
    request_url = f"{cli_args['ip_address']}/api/v{cli_args['api_version']}/projects/{cli_args['project_group']}%2F{cli_args['project']}/issues?state=closed"
    logger.info(
        f"Requesting tags for project {cli_args['project']} with URL: {request_url}"
    )
    try:
        response = requests.get(
            request_url,
            headers={"PRIVATE-TOKEN": cli_args["token"]}
            if "token" in cli_args
            else None,
            verify=cli_args["ssl"],
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as ex:
        logger.error(
            f"{get_commits_since_date.__name__} call to GitLab API failed with HTTPError: {ex}"
        )
        sys.exit(1)
    except requests.exceptions.ConnectionError as ex:
        logger.error(
            f"{get_commits_since_date.__name__} call to GitLab API failed with ConnectionError: {ex}"
        )
        sys.exit(1)

    logger.debug(response.status_code)
    logger.debug(response.json())

    return response.json()


def get_last_tagged_release_date(cli_args: dict) -> str:
    """
    Queries a specified GitLab API and returns a string containing
    the created_at date of the last tagged release.
    """
    request_url = f"{cli_args['ip_address']}/api/v{cli_args['api_version']}/projects/{cli_args['project_group']}%2F{cli_args['project']}/repository/tags"
    logger.info(
        f"Requesting tags for project {cli_args['project']} with URL: {request_url}"
    )
    try:
        response = requests.get(
            request_url,
            headers={"PRIVATE-TOKEN": cli_args["token"]}
            if "token" in cli_args
            else None,
            verify=cli_args["ssl"],
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as ex:
        logger.error(
            f"{get_commits_since_date.__name__} call to GitLab API failed with HTTPError: {ex}"
        )
        sys.exit(1)
    except requests.exceptions.ConnectionError as ex:
        logger.error(
            f"{get_commits_since_date.__name__} call to GitLab API failed with ConnectionError: {ex}"
        )
        sys.exit(1)

    logger.debug(response.status_code)
    logger.debug(response.json())

    return response.json()[0]["commit"]["created_at"]


def get_commits_since_date(date: str, cli_args: dict) -> list:
    """
    Queries a specified GitLab API and returns a JSON response containing
    all commits since a given date.
    """

    request_url = f"{cli_args['ip_address']}/api/v{cli_args['api_version']}/projects/{cli_args['project_group']}%2F{cli_args['project']}/repository/commits/?ref_name={cli_args['branch_two']}&since={date}"
    logger.info(
        f"Requesting commits on branch '{cli_args['branch_two']}' in repository '{cli_args['project']}' since date '{date}' with URL: {request_url}"
    )
    try:
        response = requests.get(
            request_url,
            headers={"PRIVATE-TOKEN": cli_args["token"]}
            if "token" in cli_args
            else None,
            verify=cli_args["ssl"],
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as ex:
        logger.error(
            f"{get_commits_since_date.__name__} call to GitLab API failed with HTTPError: {ex}"
        )
        sys.exit(1)
    except requests.exceptions.ConnectionError as ex:
        logger.error(
            f"{get_commits_since_date.__name__} call to GitLab API failed with ConnectionError: {ex}"
        )
        sys.exit(1)

    logger.debug(response.status_code)
    logger.debug(response.json())

    response_json = response.json()
    clean_response = [
        item for item in response_json if len(list(item["parent_ids"])) == 1
    ]

    return sorted(
        clean_response,
        key=lambda x: datetime.datetime.strftime(
            parser.parse(x["committed_date"]), "%Y-%m-%dT%H:%M:%S.%f"
        ),
    )
